/** 
 * A velocity controller for a single EyeBot wheel
 */

#ifndef DCVELCONTROL_H
#define DCVELCONTROL_H

#define TICKS_PER_METRE	30000

#include "PID.h"
#include "eyebot.h"

/**
 * DC Motor velocity controller
 */

typedef struct{
  MotorHandle motor;
  QuadHandle quad;

  int pos;		/* last reading from encoder */ 
  int timeScale;	/* frequency of calculation is assumed to be 100/timeScale Hz */
  int driveRate;  /* current percentage power being supplied to motor */

  PID *pid;
} DCVelControl;

void DCVelControlInit( DCVelControl **ctrlRef,
		       DeviceSemantics motor,
		       DeviceSemantics quad);

/* set the desired velocity of the wheel.  Value is in m/s */
void DCVelControlSet(DCVelControl *ctrl, float velocity);

/**
 * Returns the current setpoint of the velocity controller in metres/sec
 */
float DCVelControlGet(DCVelControl *control); 

void DCVelControlSetTimeScale(DCVelControl *ctrl, int s);

int DCVelControlSetGains(DCVelControl *control, float Kp, float Ki, float Kd);

void DCVelControlCalc(DCVelControl *ctrl);

void DCVelControlRelease(DCVelControl *control);

#endif














