/**
 * Generic PID Controller
 */

#include "PID.h"

/**
 * PID methods
 */

void PIDSetGains(PID *pid, float Kp, float Ki, float Kd) {
	pid->Kp = Kp;
	pid->Ki = Ki;
	pid->Kd = Kd;
}
		
int PIDInit(PID **pidRef) {
	*pidRef = (PID *)malloc(sizeof(PID));
	(*pidRef)->errorSeries = (float *)malloc(HISTORY*sizeof(float));

	PIDSetGains(*pidRef, DEFAULT_KP, DEFAULT_KI, DEFAULT_KD);

	(*pidRef)->setPoint = 0;
	(*pidRef)->error = 0;
	(*pidRef)->errorIndex = 0;
	(*pidRef)->errorSum = 0;

	return 0;
}

void PIDSetGains(PID *pid, float Kp, float Ki, float Kd) {
	pid->Kp = Kp;
	pid->Ki = Ki;
	pid->Kd = Kd;
}

float PIDCalc(PID *pid, float measured) {
	float feedback;
	float newError = pid->setPoint - measured;

	/* integral term calculation */ 
	pid->errorSum = pid->errorSum - pid->errorSeries[pid->errorIndex] + newError;
	pid->errorSeries[pid->errorIndex] = newError;
	pid->errorIndex = (pid->errorIndex +1) % HISTORY;	

	/* PID calculation */
	feedback = (pid->Kp * newError) +
		(pid->Ki * pid->errorSum) +
		(pid->Kd * (newError - pid->error));

	pid->error = newError;

	return feedback;
}

void PIDDestroy(PID *pid) {
	free(pid->errorSeries);
	free(pid);
}
